import logging
import time
import uuid
from logging.config import dictConfig

from .kfproducer import push_to_queue
from .eswrapper import index_error_to_es
from .loghandler import log_exception
from .config import anu_etl_wf_error_topic
from .config import es_url

log = logging.getLogger('file')


# Method to standardize and index errors for the core flow
# code: Any string that uniquely identifies an error
# message: The error message
# cause: String that explains the cause of the error
def post_error(code, message, cause):
    try:
        error = {
            "errorID": generate_error_id(),
            "code": code,
            "message": message,
            "timeStamp": eval(str(time.time()).replace('.', '')[0:13]),
            "errorType": "core-error"
        }
        if cause is not None:
            error["cause"] = str(cause)
        if es_url != 'localhost':
            index_error_to_es(error)
        return error
    except Exception as e:
        log_exception("Error Handler Failed.", None, e)
        return None


# Method to standardize, post and index errors for the workflow.
# code: Any string that uniquely identifies an error
# message: The error message
# cause: JSON or String that explains the cause of the error
# entity: Input object as received from WFM, enriched with taskID
def post_error_wf(code, message, entity, cause):
    try:
        error = {
            "errorID": generate_error_id(),
            "code": code,
            "message": message,
            "timeStamp": eval(str(time.time()).replace('.', '')[0:13]),
            "status": "FAILED",
            "errorType": "wf-error"
        }
        if cause is not None:
            error["cause"] = str(cause)
        if entity is not None:
            if 'jobID' in entity.keys():
                error["jobID"] = entity["jobID"]
            if 'taskID' in entity.keys():
                error["taskID"] = entity["taskID"]
            if 'state' in entity.keys():
                error["state"] = entity["state"]
            if 'status' in entity.keys():
                error["status"] = entity["status"]
            if 'metadata' in entity.keys():
                error["metadata"] = entity["metadata"]
        if es_url != 'localhost':
            push_to_queue(error, anu_etl_wf_error_topic)
            index_error_to_es(error)
        return error
    except Exception as e:
        log_exception("Error Handler WF Failed.", None, e)
        return None


# Method to generate error ID.
def generate_error_id():
    return str(uuid.uuid4())

# Log config
dictConfig({
    'version': 1,
    'formatters': {'default': {
        'format': '[%(asctime)s] {%(filename)s:%(lineno)d} %(threadName)s %(levelname)s in %(module)s: %(message)s',
    }},
    'handlers': {
        'info': {
            'class': 'logging.FileHandler',
            'level': 'DEBUG',
            'formatter': 'default',
            'filename': 'info.log'
        },
        'console': {
            'class': 'logging.StreamHandler',
            'level': 'DEBUG',
            'formatter': 'default',
            'stream': 'ext://sys.stdout',
        }
    },
    'loggers': {
        'file': {
            'level': 'DEBUG',
            'handlers': ['info', 'console'],
            'propagate': ''
        }
    },
    'root': {
        'level': 'DEBUG',
        'handlers': ['info', 'console']
    }
})



import logging
import time
import uuid
import inspect
from logging.config import dictConfig

from .eswrapper import index_audit_to_es

log = logging.getLogger('file')
from .config import es_url
from .config import is_info_enabled
from .config import is_debug_enabled
from .config import is_error_enabled
from .config import is_exc_enabled


# Method to log and index INFO level logs
# message: The message to be logged
# entity: Input object as received from WFM, enriched with taskID
def log_info(message, entity):
    log.info(message)
    if es_url != 'localhost' and is_info_enabled:
        try:
            previous_frame = inspect.currentframe().f_back
            (filename, line_number,
             function_name, lines, index) = inspect.getframeinfo(previous_frame)
            audit = {
                "auditID": generate_error_id(),
                "methodName": function_name,
                "fileName": filename,
                "lineNo": line_number,
                "message": str(message),
                "timeStamp": eval(str(time.time()).replace('.', '')[0:13]),
                "auditType": "INFO"
            }
            audit_enriched = enrich_entity_details(audit, entity)
            if audit_enriched is not None:
                audit = audit_enriched
            index_audit_to_es(audit)
        except Exception as e:
            log.exception("Anuvaad Auditor INFO failed.", e)
            return None


# Method to log and index DEBUG level logs
# message: The message to be logged
# entity: Input object as received from WFM, enriched with taskID
def log_debug(message, entity):
    log.debug(message)
    if es_url != 'localhost' and is_debug_enabled:
        try:
            previous_frame = inspect.currentframe().f_back
            (filename, line_number,
             function_name, lines, index) = inspect.getframeinfo(previous_frame)
            audit = {
                "auditID": generate_error_id(),
                "methodName": function_name,
                "fileName": filename,
                "lineNo": line_number,
                "message": str(message),
                "timeStamp": eval(str(time.time()).replace('.', '')[0:13]),
                "auditType": "DEBUG"
            }
            audit_enriched = enrich_entity_details(audit, entity)
            if audit_enriched is not None:
                audit = audit_enriched
            index_audit_to_es(audit)
        except Exception as e:
            log.exception("Anuvaad Auditor DEBUG failed.", e)
            return None


# Method to log and index EXCEPTION level logs
# message: The message to be logged
# entity: Input object as received from WFM, enriched with taskID.
# exc: Exception object
def log_exception(message, entity, exc):
    log.exception(message)
    if es_url != 'localhost' and is_exc_enabled:
        try:
            previous_frame = inspect.currentframe().f_back
            (filename, line_number,
             function_name, lines, index) = inspect.getframeinfo(previous_frame)
            audit = {
                "auditID": generate_error_id(),
                "methodName": function_name,
                "fileName": filename,
                "lineNo": line_number,
                "message": str(message),
                "timeStamp": eval(str(time.time()).replace('.', '')[0:13]),
                "auditType": "EXCEPTION"
            }
            audit_enriched = enrich_entity_details(audit, entity)
            if audit_enriched is not None:
                audit = audit_enriched
            if exc is not None:
                audit["cause"] = str(exc)
            index_audit_to_es(audit)
        except Exception as e:
            log.exception("Anuvaad Auditor EXCEPTION failed.", e)
            return None


# Method to log and index ERROR level logs
# message: The message to be logged
# entity: Input object as received from WFM, enriched with taskID.
# exc: Exception object
def log_error(message, entity, exc):
    log.error(message)
    if es_url != 'localhost' and is_error_enabled:
        try:
            previous_frame = inspect.currentframe().f_back
            (filename, line_number,
             function_name, lines, index) = inspect.getframeinfo(previous_frame)
            audit = {
                "auditID": generate_error_id(),
                "methodName": function_name,
                "fileName": filename,
                "lineNo": line_number,
                "message": str(message),
                "timeStamp": eval(str(time.time()).replace('.', '')[0:13]),
                "auditType": "ERROR"
            }
            audit_enriched = enrich_entity_details(audit, entity)
            if audit_enriched is not None:
                audit = audit_enriched
            if exc is not None:
                audit["cause"] = str(exc)
            index_audit_to_es(audit)
        except Exception as e:
            log.exception("Anuvaad Auditor ERROR failed.", e)
            return None


# Enriches the audit object with entity related data
def enrich_entity_details(audit, entity):
    try:
        if entity is not None:
            if 'jobID' in entity.keys():
                audit["entityID"] = entity["jobID"]
            else:
                audit["entityID"] = "JOB-ID-NA"
            if 'taskID' in entity.keys():
                audit["taskID"] = entity["taskID"]
            else:
                audit["taskID"] = "TASK-ID-NA"
            if 'metadata' in entity.keys():
                metadata = entity["metadata"]
                if 'sessionID' in metadata.keys():
                    audit["sessionID"] = metadata["sessionID"]
                else:
                    audit["sessionID"] = "SE-ID-NA"
                if 'userID' in metadata.keys():
                    audit["userID"] = metadata["userID"]
                else:
                    audit["userID"] = "USR-ID-NA"
                if 'module' in metadata.keys():
                    audit["module"] = metadata["module"]
                else:
                    audit["module"] = "MOD-NA"
                if 'requestID' in metadata.keys():
                    audit["requestID"] = metadata["requestID"]
                else:
                    audit["requestID"] = "REQ-ID-NA"
            else:
                audit["sessionID"] = "SE-ID-NA"
                audit["userID"] = "USR-ID-NA"
                audit["module"] = "MOD-NA"
                audit["requestID"] = "MOD-NA"
        return audit
    except Exception as e:
        log.exception("Exception while enriching with entity!", e)
        return None


# Audit ID generator
def generate_error_id():
    return str(uuid.uuid4())


# Log config
dictConfig({
    'version': 1,
    'formatters': {'default': {
        'format': '[%(asctime)s] {%(filename)s:%(lineno)d} %(threadName)s %(levelname)s in %(module)s: %(message)s',
    }},
    'handlers': {
        'info': {
            'class': 'logging.FileHandler',
            'level': 'DEBUG',
            'formatter': 'default',
            'filename': 'info.log'
        },
        'console': {
            'class': 'logging.StreamHandler',
            'level': 'DEBUG',
            'formatter': 'default',
            'stream': 'ext://sys.stdout',
        }
    },
    'loggers': {
        'file': {
            'level': 'DEBUG',
            'handlers': ['info', 'console'],
            'propagate': ''
        }
    },
    'root': {
        'level': 'DEBUG',
        'handlers': ['info', 'console']
    }
})



from Logs_Auditor.errorhandler import post_error
from Logs_Auditor.errorhandler import post_error_wf
from Logs_Auditor.loghandler import log_info
from Logs_Auditor.loghandler import log_debug
from Logs_Auditor.loghandler import log_error
from Logs_Auditor.loghandler import log_exception



import logging
from datetime import datetime
from logging.config import dictConfig

from elasticsearch import Elasticsearch
from .kfproducer import push_to_queue
from .config import es_url
from .config import es_core_error_index
from .config import es_wf_error_index
from .config import es_audit_index
from .config import anuvaad_log_topic

log = logging.getLogger('file')

es_core_error_index_test = "anuvaad-etl-core-errors-test-v1"
es_wf_error_index_test = "anuvaad-etl-wf-errors-test-v1"
es_audit_index_test = "anuvaad-etl-audit-test-v1"

# Method to instantiate the elasticsearch client.
def instantiate_es_client():
    es_client = Elasticsearch([es_url])
    return es_client


# Method to index errors on to elasticsearch.
def index_error_to_es(index_obj):
    try:
        # es = instantiate_es_client()
        id = index_obj["errorID"]
        if index_obj["errorType"] == "core-error":
            in_name = es_core_error_index
        else:
            in_name = es_wf_error_index
        index_obj = add_timestamp_field(index_obj)
        # es.index(index=in_name, id=id, body=index_obj)
        push_to_queue(index_obj,anuvaad_log_topic)
    except Exception as e:
        log.exception("Indexing FAILED for errorID: " + index_obj["errorID"], e)


# Method to index audit details onto elasticsearch
def index_audit_to_es(index_obj):
    try:
        # es = instantiate_es_client()
        id = index_obj["auditID"]
        index_obj = add_timestamp_field(index_obj)
        # es.index(index=es_audit_index, id=id, body=index_obj)
        push_to_queue(index_obj,anuvaad_log_topic)

    except Exception as e:
        log.exception("Indexing FAILED for auditID: " + index_obj["auditID"], e)


# Method to generate timestamp in the format es expects per index object.
def add_timestamp_field(error):
    epoch_short = eval((str(error["timeStamp"])[0:10]))
    error["@timestamp"] = datetime.fromtimestamp(epoch_short).isoformat()
    return error




# Log config
dictConfig({
    'version': 1,
    'formatters': {'default': {
        'format': '[%(asctime)s] {%(filename)s:%(lineno)d} %(threadName)s %(levelname)s in %(module)s: %(message)s',
    }},
    'handlers': {
        'info': {
            'class': 'logging.FileHandler',
            'level': 'DEBUG',
            'formatter': 'default',
            'filename': 'info.log'
        },
        'console': {
            'class': 'logging.StreamHandler',
            'level': 'DEBUG',
            'formatter': 'default',
            'stream': 'ext://sys.stdout',
        }
    },
    'loggers': {
        'file': {
            'level': 'DEBUG',
            'handlers': ['info', 'console'],
            'propagate': ''
        }
    },
    'root': {
        'level': 'DEBUG',
        'handlers': ['info', 'console']
    }
})


import os

kafka_bootstrap_server_host = os.environ.get('KAFKA_BOOTSTRAP_SERVER_HOST', 'localhost:9092')

anu_etl_wf_error_topic = os.environ.get('KAFKA_ANUVAAD_ETL_WF_ERROR_TOPIC', 'anuvaad-etl-wf-errors-v1')
anuvaad_log_topic = os.environ.get('KAFKA_ANUVAAD_LOGS_TOPIC' , 'anuvaad-logs-v1')
es_url = os.environ.get('ANUVAAD_DP_ES_URL', 'localhost')

es_core_error_index = os.environ.get('ANUVAAD_DP_ES_CORE_ERROR_INDEX', 'anuvaad-dp-errors-core-v1')
es_wf_error_index = os.environ.get('ANUVAAD_DP_ES_WF_ERROR_INDEX', 'anuvaad-dp-errors-wf-v1')
es_audit_index = os.environ.get('ANUVAAD_DP_ES_AUDIT_INDEX', 'anuvaad-dp-audit-v1')

is_debug_enabled = os.environ.get('AUDITOR_LOG_DEBUG_ENABLED', True)
is_info_enabled = os.environ.get('AUDITOR_LOG_INFO_ENABLED', True)
is_error_enabled = os.environ.get('AUDITOR_LOG_ERROR_ENABLED', True)
is_exc_enabled = os.environ.get('AUDITOR_LOG_EXC_ENABLED', True)



import json
import logging

import os
from logging.config import dictConfig
from .config import kafka_bootstrap_server_host
from kafka import KafkaProducer


log = logging.getLogger('file')


def instantiate():
    producer = KafkaProducer(bootstrap_servers=list(str(kafka_bootstrap_server_host).split(",")),
                             api_version=(1, 0, 0),
                             value_serializer=lambda x: json.dumps(x).encode('utf-8'))
    return producer


# Method to push records to a topic in the kafka queue
def push_to_queue(object_in, topic):
    producer = instantiate()
    try:
        producer.send(topic, value=object_in)
        log.info("Pushing to the topic: " + topic)
        producer.flush()
    except Exception as e:
        log.exception("Exception while producing: " + str(e))


# Log config
dictConfig({
    'version': 1,
    'formatters': {'default': {
        'format': '[%(asctime)s] {%(filename)s:%(lineno)d} %(threadName)s %(levelname)s in %(module)s: %(message)s',
    }},
    'handlers': {
        'info': {
            'class': 'logging.FileHandler',
            'level': 'DEBUG',
            'formatter': 'default',
            'filename': 'info.log'
        },
        'console': {
            'class': 'logging.StreamHandler',
            'level': 'DEBUG',
            'formatter': 'default',
            'stream': 'ext://sys.stdout',
        }
    },
    'loggers': {
        'file': {
            'level': 'DEBUG',
            'handlers': ['info', 'console'],
            'propagate': ''
        }
    },
    'root': {
        'level': 'DEBUG',
        'handlers': ['info', 'console']
    }
})
